package jp.gr.java_conf.skrb.game.trivia.server;

import java.io.FileInputStream;
import java.io.IOException;
import java.net.Socket;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import jp.gr.java_conf.skrb.game.trivia.util.Counter;

public class TriviaServer {
    protected Map<Socket, Entry> clients;
    private Map<Socket, String> waitingClients;
    private Map<Socket, Entry> closedClients;
    private ServerCommunicator communicator;

    protected TotalCounter counter;
    private ServerView view;

    private Properties properties;

    protected static class Entry {
        private String name;
        private Counter counter;

        Entry(String name, Counter counter) {
            this.name = name;
            this.counter = counter;
        }

        public String getName() {
            return name;
        }

        public Counter getCounter() {
            return counter;
        }
    }

    public TriviaServer() {
        clients = new HashMap<Socket, Entry>();
        waitingClients = new HashMap<Socket, String>();
        closedClients = new HashMap<Socket, Entry>();
        counter = new TotalCounter(0);

        initProperties();
        initGUI();

        try {
            communicator = new ServerCommunicator(this);
        } catch (IOException ex) {
            ex.printStackTrace();
            System.exit(1);
        }

        accept();
    }

    public void accept() {
        try {
            communicator.accept();
        } catch (IOException ex) {
            ex.printStackTrace();
            System.exit(1);
        }
    }

    public void connectClient(Socket socket, String name) {
        if (counter.getCount() == 0) {
            entryClient(socket, name);
        } else {
            registerClient(socket, name);
        }
    }

    public void disconnectClient(Socket socket) {
        Entry entry = clients.get(socket);
        closedClients.put(socket, entry);
    }

    public void readData(Socket socket, int count) {
        Entry entry = (Entry)clients.get(socket);

        if (count != -1) {
            counter.increment();
            entry.getCounter().increment();

            view.updateClient(socket.hashCode());
        }
    }

    public void reset() {
        counter.reset();
        updateClients();

        for (Entry entry : clients.values()) {
            entry.getCounter().reset();
        }
        view.resetClients();
        
        sendReset();
    }

    private void initProperties() {
        properties = new Properties();

        try {
            FileInputStream stream = new FileInputStream("server.properties");
            properties.loadFromXML(stream);
            stream.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private void initGUI() {
        view = new DefaultServerView(this, counter, waitingClients.values(), properties);
    }

    private Entry addClient(Integer id, String name) {
        Counter counter = new Counter();

        view.addClient(id, name, counter);

        return new Entry(name, counter);
    }

    private void entryClient(Socket socket, String name) {
        Entry entry = clients.get(socket);
        if (entry == null) {
            counter.incrementNumber();
            clients.put(socket, addClient(socket.hashCode(), name));
        }
        communicator.sendReset(socket);
    }

    private void updateClients() {
        Iterator<Socket> it = waitingClients.keySet().iterator();

        while (it.hasNext()) {
            Socket socket = it.next();
            counter.incrementNumber();
            clients.put(socket, addClient(socket.hashCode(), waitingClients.get(socket)));
            it.remove();
        }

        view.updateWaitingList(waitingClients.values());

        it = closedClients.keySet().iterator();
        while (it.hasNext()) {
            Socket socket = it.next();
            Entry entry = closedClients.get(socket);

            view.removeClient(socket.hashCode());

            clients.remove(socket);
            
            counter.decrementNumber();
            it.remove();
        }
    }

    private void registerClient(Socket socket, String name) {
        waitingClients.put(socket, name);

        view.updateWaitingList(waitingClients.values());
    }

    private void sendReset() {
        for (Socket socket: clients.keySet()) {
            communicator.sendReset(socket);
        }
    }

    public static void main(String[] args) {
        new TriviaServer();
    }
}
